from unittest import TestCase

from django.contrib.auth.models import AnonymousUser
from django.contrib.comments.models import Comment
from django.contrib.contenttypes.models import ContentType
from django.template import Template, Context
from django.test.client import RequestFactory
from likes.middleware import SecretBallotUserIpUseragentMiddleware
from likes.views import can_vote_test, like
from secretballot import views
from secretballot.models import Vote


class BaseClassifierTestCase(object):
    def setUp(self):
        if 'CLASSIFIER_CONFIG' not in self.config:
            self.config['CLASSIFIER_CONFIG'] = {}

        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.clear()
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )

    def test_init(self):
        # State's counts should be loaded in the classifier.
        # On initial creation of classifier counts are 0.
        self.failUnlessEqual(self.classifier.nspam, 0)
        self.failUnlessEqual(self.classifier.nham, 0)

        # On subsequent load counts are loaded from state.
        self.classifier.nspam = 10
        self.classifier.nham = 20
        self.classifier.store()
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.failUnlessEqual(self.classifier.nspam, 10)
        self.failUnlessEqual(self.classifier.nham, 20)

    def test_store(self):
        state = self.classifier.get_state()
        self.failIfEqual(
            state.spam_count,
            50,
            'Internal checking test is not testing existing values'
        )
        self.failIfEqual(
            state.ham_count,
            100,
            'Internal checking test is not testing existing values'
        )

        # On store classifier counts should be saved to DB.
        self.classifier.nspam = 50
        self.classifier.nham = 100
        self.classifier.store()
        state = self.classifier.get_state()
        self.failUnlessEqual(state.spam_count, 50)
        self.failUnlessEqual(state.ham_count, 100)


class UtilsTestCase(TestCase):
    def setUp(self):
        from moderator import utils
        self.utils = utils

    def test_classify_comment(self):
        spam_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="very bad spam"
        )
        ham_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome tasty ham"
        )
        unsure_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="tasty spam"
        )
        generic_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="foo bar"
        )
        abusive_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="abusive comment"
        )

        for i in range(0, 3):
            Vote.objects.create(
                content_type=ContentType.objects.get_for_model(Comment),
                object_id=abusive_comment.id,
                token=i,
                vote=-1
            )

        # Providing unsure should create unsure classification without
        # training.
        classified_comment = self.utils.classify_comment(
            generic_comment,
            'unsure'
        )
        self.failUnlessEqual(classified_comment.cls, 'unsure')

        # Providing reported should create reported classification without
        # training, comment should be removed.
        classified_comment = self.utils.classify_comment(
            generic_comment,
            'reported'
        )
        self.failUnlessEqual(classified_comment.cls, 'reported')
        self.failUnless(classified_comment.comment.is_removed)

        # Without providing a class but with user abuse reports more or equal
        # to cutoff should create reported classification without training,
        # comment should be removed.
        classified_comment = self.utils.classify_comment(abusive_comment)
        self.failUnlessEqual(classified_comment.cls, 'reported')
        self.failUnless(classified_comment.comment.is_removed)

        # Providing ham class should create ham classification and training,
        # comment should not be removed.
        classified_comment = self.utils.classify_comment(ham_comment, 'ham')
        self.failUnlessEqual(classified_comment.cls, 'ham')
        self.failIf(classified_comment.comment.is_removed)

        # Providing spam class should create spam classification and training,
        # comment should be removed.
        classified_comment = self.utils.classify_comment(spam_comment, 'spam')
        self.failUnlessEqual(classified_comment.cls, 'spam')
        self.failUnless(classified_comment.comment.is_removed)

        # Spammy comment should now be correctly classified automatically
        # without any training, should be removed.
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="bad spam"
        )
        comment.total_downvotes = 0
        classified_comment = self.utils.classify_comment(comment)
        self.failUnlessEqual(classified_comment.cls, 'unsure')

        # Hammy comment should now be correctly classified automatically
        # without any training, should not be removed.
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="tasty ham"
        )
        comment.total_downvotes = 0
        classified_comment = self.utils.classify_comment(comment)
        self.failUnlessEqual(classified_comment.cls, 'unsure')
        self.failIf(classified_comment.comment.is_removed)

        # Hammy spammy comment should now be correctly classified automatically
        # as unsure without any training, should not be removed.
        classified_comment = self.utils.classify_comment(unsure_comment)
        self.failUnlessEqual(classified_comment.cls, 'unsure')
        self.failIf(classified_comment.comment.is_removed)

        # Should raise exception with unkown cls.
        self.assertRaises(Exception, self.utils.classify_comment,
                          unsure_comment, 'unknown_cls')

        classified_comment = self.utils.classify_comment(spam_comment, 'spam')


class InclusionTagsTestCase(TestCase):

    def test_report_comment_abuse(self):
        # Prepare context.
        context = Context()
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        request.META['HTTP_USER_AGENT'] = 'testing_agent'
        request.secretballot_token = SecretBallotUserIpUseragentMiddleware().\
            generate_token(request)
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="abuse report testing comment"
        )
        context['request'] = request
        context['comment'] = comment

        # Without having actioned anything on the comment the
        # Report Abuse action should be rendered.
        out = Template("{% load moderator_inclusion_tags %}"
                       "{% report_comment_abuse comment %}").render(context)
        self.failUnless('Report Abuse' in out)

        # Like a comment.
        views.vote(
            request,
            content_type='.'.join((comment._meta.app_label,
                                   comment._meta.module_name)),
            object_id=comment.id,
            vote=1,
            redirect_url='/',
            can_vote_test=can_vote_test
        )

        # Reset previous like and test it applied.
        Vote.objects.all().delete()

        # Without having actioned anything on the comment the
        # Report Abuse action should be rendered.
        out = Template("{% load moderator_inclusion_tags %}"
                       "{% report_comment_abuse comment %}").render(context)
        self.failUnless('Report Abuse' in out)

        # Dislike/report an abuse comment.
        views.vote(
            request,
            content_type='.'.join((comment._meta.app_label,
                                   comment._meta.module_name)),
            object_id=comment.id,
            vote=-1,
            redirect_url='/',
            can_vote_test=can_vote_test
        )

        self.assertEqual(Vote.objects.all().count(), 1)

        #repeat votes should not count
        views.vote(
            request,
            content_type='.'.join((comment._meta.app_label,
                                   comment._meta.module_name)),
            object_id=comment.id,
            vote=-1,
            redirect_url='/',
            can_vote_test=can_vote_test
        )

        self.assertEqual(Vote.objects.all().count(), 1)

    def test_report_comment_abuse_signal(self):
        # Prepare context.
        context = Context()
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        request.META['HTTP_REFERER'] = '/'
        request.META['HTTP_USER_AGENT'] = 'testing_agent'
        request.secretballot_token = SecretBallotUserIpUseragentMiddleware().\
            generate_token(request)
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="abuse report testing comment"
        )
        context['request'] = request
        context['comment'] = comment
        content_type = '-'.join((comment._meta.app_label,
                                 comment._meta.module_name))

        # Reset previous like and test it applied.
        Vote.objects.all().delete()

        # Report an abuse comment - 1st
        like(
            request,
            content_type=content_type,
            id=comment.id,
            vote=-1
        )
        self.assertEqual(Vote.objects.all().count(), 1)
        self.failIf(Comment.objects.get(pk=comment.pk).is_removed)

        # Report an abuse comment - 2nd
        request.META['HTTP_USER_AGENT'] = 'testing_agent_2'
        request.secretballot_token = SecretBallotUserIpUseragentMiddleware().\
            generate_token(request)
        like(
            request,
            content_type=content_type,
            id=comment.id,
            vote=-1,
        )
        self.assertEqual(Vote.objects.all().count(), 2)
        self.failIf(Comment.objects.get(pk=comment.pk).is_removed)

        # Report an abuse comment - 3rd
        request.META['HTTP_USER_AGENT'] = 'testing_agent_3'
        request.secretballot_token = SecretBallotUserIpUseragentMiddleware().\
            generate_token(request)
        like(
            request,
            content_type=content_type,
            id=comment.id,
            vote=-1,
        )
        self.assertEqual(Vote.objects.all().count(), 3)
        self.failUnless(Comment.objects.get(pk=comment.pk).is_removed)



from django.contrib import admin
from django.contrib.admin.templatetags.admin_static import static
from django.contrib.admin.util import unquote
from django.contrib.admin.views.main import ChangeList
from django.contrib.comments.admin import CommentsAdmin as DjangoCommentsAdmin
from django.contrib.comments.models import Comment
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.db import transaction
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import defaultfilters
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_protect
from moderator import models, utils

csrf_protect_m = method_decorator(csrf_protect)


class CannedReplyAdmin(admin.ModelAdmin):
    list_display = ('comment', 'site', )
    list_filter = ('site', )


class CommentReplyAdmin(admin.ModelAdmin):
    raw_id_fields = ("replied_to_comments", )
    exclude = ("reply_comments", )
    fk_name = 'replied_to_comments'

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        """
        Limit canned reply options to those with same site as comment.
        """
        field = super(CommentReplyAdmin, self).\
            formfield_for_foreignkey(db_field, request, **kwargs)
        comment_id = request.GET.get(self.fk_name, None)

        if db_field.name == 'canned_reply' and comment_id:
            comment_site = Comment.objects.get(id=comment_id).site
            field.queryset = field.queryset.filter(Q(site=comment_site) |
                                                   Q(site__isnull=True))
        return field

    def response_add(self, request, obj, post_url_continue=None):
        if 'admin_redirect' in request.session:
            redirect = request.session['admin_redirect']
            del request.session["admin_redirect"]
            return HttpResponseRedirect(redirect)
        else:
            return super(CommentReplyAdmin, self).response_add(request, obj, post_url_continue)


class CommentAdmin(DjangoCommentsAdmin):
    list_display = (
        'comment_text',
        'moderator_reply',
        'content',
        '_user',
        'submit_date',
    )
    actions = ['add_moderator_reply', 'mark_ham', 'mark_spam', 'mark_spam_with_reply', ]
    date_hierarchy = None

    def queryset(self, request):
        """
        Exclude replies from listing since they are displayed inline as part of listing.

        For proxy models with cls apptribute limit comments to those classified as cls.
        """
        qs = super(CommentAdmin, self).queryset(request)
        qs = qs.filter(Q(user__is_staff=False) | Q(user__isnull=True), is_removed=False)
        cls = getattr(self, 'cls', None)
        if cls:
            qs = qs.filter(classifiedcomment__cls=self.cls)
        return qs.select_related('user', 'content_type')

    def add_moderator_reply(self, modeladmin, request, queryset):
        selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
        request.session["admin_redirect"] = request.get_full_path()
        return HttpResponseRedirect(reverse('admin:moderator_commentreply_add') + '?replied_to_comments=%s' % ",".join(selected))
    add_moderator_reply.short_description = "Add moderator reply"

    def mark_spam(self, modeladmin, request, queryset):
        for comment in queryset:
            utils.classify_comment(comment, cls='spam')
        self.message_user(request, "%s comment(s) successfully marked as spam." % queryset.count())
    mark_spam.short_description = "Mark selected comments as spam"

    def mark_spam_with_reply(self, modeladmin, request, queryset):
        self.mark_spam(modeladmin, request, queryset)
        return self.add_moderator_reply(modeladmin, request, queryset)
    mark_spam_with_reply.short_description = "Mark selected comments as spam, replying at the same time."

    def mark_ham(self, modeladmin, request, queryset):
        for comment in queryset:
            utils.classify_comment(comment, cls='ham')
        self.message_user(request, "%s comment(s) successfully marked as ham." % queryset.count())
    mark_ham.short_description = "Mark selected comments as ham"

    def get_actions(self, request):
        actions = {}
        for action in self.actions:
            actions[action] = (
                getattr(self, action),
                action,
                getattr(self, action).short_description,
            )
        return actions

    def get_changelist(self, request):
        class ModeratorChangeList(ChangeList):
            def get_query_set(self, request):
                """
                Used by AdminModeratorMixin.moderate_view to somewhat hackishly limit comments to only those
                for the object under review, but only if an obj attribute is found on request
                (which means the mixin is being applied and we are not on the standard changelist_view).
                """
                qs = super(ModeratorChangeList, self).get_query_set(request)
                obj = getattr(request, 'obj', None)
                if obj:
                    ct = ContentType.objects.get_for_model(obj)
                    qs = qs.filter(content_type=ct, object_pk=obj.pk)
                return qs

            def get_results(self, request):
                """
                Create a content_type map to individual objects through their id's to avoid
                additional per object queries for generic relation lookup (used in CommentAdmin.content
                method).

                Also create a comment_reply map to avoid additional reply lookups per comment
                object (used in CommentAdmin.moderator_reply method)
                """
                super(ModeratorChangeList, self).get_results(request)
                comment_ids = []
                object_pks = []

                results = list(self.result_list)
                for obj in results:
                    comment_ids.append(obj.id)
                    object_pks.append(obj.object_pk)

                ct_map = {}
                for obj in results:
                    if obj.content_type not in ct_map:
                        ct_map.setdefault(obj.content_type, {})
                        for content_obj in obj.content_type.model_class()._default_manager.filter(pk__in=object_pks):
                            ct_map[obj.content_type][content_obj.id] = content_obj
                self.model_admin.ct_map = ct_map

                comment_replies = {}
                for comment_reply in models.CommentReply.objects.filter(replied_to_comments__in=comment_ids):
                    for replied_to_comment in comment_reply.replied_to_comments.all():
                        comment_replies[replied_to_comment.id] = {
                            'id': comment_reply.id,
                            'comment': comment_reply.comment_text,
                        }
                self.model_admin.comment_replies = comment_replies

        return ModeratorChangeList

    def content(self, obj, *args, **kwargs):
        content_type = obj.content_type
        content = self.ct_map[content_type][int(obj.object_pk)]
        url = reverse('admin:%s_%s_moderate' % (content_type.app_label,
                                              content_type.model),
                      args=(content.id,))

        return '<a href="%s">%s</a>' % (url, content)
    content.allow_tags = True

    def moderator_reply(self, obj):
        if obj.id in self.comment_replies:
            reply = self.comment_replies[obj.id]
            change_icon = '<img src="%s" alt="change" />' % static('admin/img/icon_changelink.gif')
            return '%s <a href="%s" target="_blank">%s</a>' % (
                change_icon,
                reverse('admin:moderator_commentreply_change', args=(reply['id'], )),
                reply['comment']
            )
        else:
            add_icon = '<img src="%s" alt="add" />' % static('admin/img/icon_addlink.gif')
            return '%s <a href="%s?replied_to_comments=%s" class="add-another" id="moderator_reply_comments_%s" onclick="return showAddAnotherPopup(this);">Add reply</a>' % (
                add_icon,
                reverse('admin:moderator_commentreply_add'),
                obj.pk,
                obj.pk
            )
    moderator_reply.allow_tags = True

    def comment_text(self, obj):
        return '<a href="%s">%s</a>' % (
            reverse(
                'admin:%s_%s_change' % (obj._meta.app_label, obj._meta.module_name),
                args=(obj.id, )
            ),
            defaultfilters.linebreaks(obj.comment)
        )
    comment_text.short_description = 'Comment'
    comment_text.allow_tags = True

    def _user(self, obj):
        url = reverse('admin:auth_user_change', args=(obj.user.id,))
        return '<a href="%s">%s</a>' % (url, obj.user)
    _user.allow_tags = True
    _user.short_description = 'User'


class HamCommentAdmin(CommentAdmin):
    cls = 'ham'
    actions = ['add_moderator_reply', 'mark_spam', 'mark_spam_with_reply', ]
    raw_id_fields = ('user', )


class ReportedCommentAdmin(CommentAdmin):
    cls = 'reported'
    actions = ['add_moderator_reply', 'mark_ham', 'mark_spam', 'mark_spam_with_reply', ]


class SpamCommentAdmin(CommentAdmin):
    cls = 'spam'
    actions = ['mark_ham', ]


class UnsureCommentAdmin(CommentAdmin):
    cls = 'unsure'
    actions = ['add_moderator_reply', 'mark_ham', 'mark_spam', 'mark_spam_with_reply', ]


class AdminModeratorMixin(object):
    @csrf_protect_m
    @transaction.commit_on_success
    def change_view(self, request, object_id, form_url='', extra_context=None):
        """
        Override change view to add extra context enabling moderate tool.
        """
        context = {
            'has_moderate_tool': True
        }
        if extra_context:
            context.update(extra_context)
        return super(AdminModeratorMixin, self).change_view(
            request=request,
            object_id=object_id,
            form_url=form_url,
            extra_context=context
        )

    def get_urls(self):
        """
        Add aditional moderate url.
        """
        from django.conf.urls import patterns, url
        urls = super(AdminModeratorMixin, self).get_urls()
        info = self.model._meta.app_label, self.model._meta.module_name
        return patterns('',
            url(r'^(.+)/moderate/$',
                self.admin_site.admin_view(self.moderate_view),
                name='%s_%s_moderate' % info),
        ) + urls

    def moderate_view(self, request, object_id, extra_context=None):
        """
        Handles moderate object tool through a somewhat hacky changelist view
        whose queryset is altered via CommentAdmin.get_changelist to only list
        comments for the object under review.
        """
        opts = self.model._meta
        app_label = opts.app_label

        view = CommentAdmin(model=Comment, admin_site=self.admin_site)

        view.list_filter = ()
        view.list_display = (
            'comment_text',
            'moderator_reply',
            '_user',
            'submit_date',
        )

        model = self.model
        obj = get_object_or_404(model, pk=unquote(object_id))
        request.obj = obj
        view.change_list_template = self.change_list_template or [
            'admin/%s/%s/moderate.html' % (app_label, opts.object_name.lower()),
            'admin/%s/moderate.html' % app_label,
            'admin/moderate.html'
        ]
        orig_has_change_permission = self.has_change_permission(request, obj)
        if not orig_has_change_permission:
            raise PermissionDenied
        extra_context = {
            'opts': opts,
            'original': obj,
            'orig_has_change_permission': orig_has_change_permission,
        }
        return view.changelist_view(request, extra_context)


admin.site.register(models.CannedReply, CannedReplyAdmin)
admin.site.unregister(Comment)
admin.site.register(Comment, CommentAdmin)
admin.site.register(models.CommentReply, CommentReplyAdmin)
admin.site.register(models.HamComment, HamCommentAdmin)
admin.site.register(models.ReportedComment, ReportedCommentAdmin)
admin.site.register(models.SpamComment, SpamCommentAdmin)
admin.site.register(models.UnsureComment, UnsureCommentAdmin)



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from moderator.constants import DEFAULT_CONFIG
from secretballot.models import Vote
from moderator import models


def classify_comment(comment, cls=None):
    """
    If 'reported' class is provided no training occures, the comment's class
    is simply set as such and removed.

    If no class is provided a lookup is done to see if the comment has been
    reported by users as abusive. If indicated as abusive class is set
    as 'reported'and the comment being removed.
    """
    if cls not in ['spam', 'ham', 'unsure', 'reported', None]:
        raise Exception("Unrecognized classifications.")

    classified_comment, cr = models.ClassifiedComment.objects.get_or_create(
        comment=comment
    )

    if cls == 'spam' and classified_comment.cls != 'spam':
        comment.is_removed = True
        comment.save()
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls == 'ham' and classified_comment.cls != 'ham':
        comment.is_removed = False
        comment.save()
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls == 'unsure' and classified_comment.cls != 'unsure':
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls == 'reported' and classified_comment.cls != 'reported':
        comment.is_removed = True
        comment.save()
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls is None:
        cls = 'unsure'
        comment_content_type = ContentType.objects.get_for_model(comment)
        moderator_settings = getattr(settings, 'MODERATOR', DEFAULT_CONFIG)
        if Vote.objects.filter(
            content_type=comment_content_type,
            object_id=comment.id,
            vote=-1
        ).count() >= moderator_settings['ABUSE_CUTOFF']:
            cls = 'reported'
            comment.is_removed = True
            comment.save()
            classified_comment.cls = cls
            classified_comment.save()
            return classified_comment
        else:
            comment.is_removed = cls == 'spam'
            comment.save()
            classified_comment.cls = cls
            classified_comment.save()
            return classified_comment

    return classified_comment



from datetime import timedelta

from django.conf import settings
from django.contrib.comments.models import Comment
from django.db import models
from django.db.models.signals import m2m_changed, post_save, pre_delete
from django.dispatch import receiver
from moderator.constants import CLASS_CHOICES
from likes.signals import object_liked
import secretballot


COMMENT_MAX_LENGTH = getattr(settings, 'COMMENT_MAX_LENGTH', 3000)


class CannedReply(models.Model):
    comment = models.TextField(max_length=COMMENT_MAX_LENGTH)
    site = models.ForeignKey(
        'sites.Site',
        blank=True,
        null=True
    )

    class Meta:
        verbose_name_plural = 'Canned replies'

    def __unicode__(self):
        return self.comment


class ClassifiedComment(models.Model):
    comment = models.ForeignKey('comments.Comment')
    cls = models.CharField(
        'Class',
        max_length=64,
        choices=CLASS_CHOICES
    )

    class Meta:
        ordering = ['-comment__submit_date', ]

    def __unicode__(self):
        return self.cls.title()


class CommentReply(models.Model):
    user = models.ForeignKey(
        'auth.User',
        limit_choices_to={'is_staff': True}
    )
    canned_reply = models.ForeignKey(
        'moderator.CannedReply',
        help_text='Select a canned reply or otherwise enter '
                  'a custom comment below.',
        blank=True,
        null=True,
    )
    comment = models.TextField(
        max_length=COMMENT_MAX_LENGTH,
        help_text='Enter a custom comment (only used if no '
                  'canned reply is selected).',
        blank=True,
        null=True,
    )
    replied_to_comments = models.ManyToManyField(
        'comments.Comment',
        related_name='replied_to_comments_set',
        help_text='Comment on which this reply applies.'
    )
    reply_comments = models.ManyToManyField(
        'comments.Comment',
        related_name='reply_comments_set',
        blank=True,
        null=True,
        help_text='Generated reply comments.'
    )

    class Meta:
        verbose_name_plural = 'Comment replies'

    def __unicode__(self):
        return "%s: %s..." % (
            self.user.username,
            self.comment[:50]
        )

    @property
    def comment_text(self):
        if self.canned_reply:
            return self.canned_reply.comment
        if self.comment:
            return self.comment


# Proxy models for admin display.
class HamComment(Comment):
    class Meta:
        proxy = True


class ReportedComment(Comment):
    class Meta:
        proxy = True


class SpamComment(Comment):
    class Meta:
        proxy = True


class UnsureComment(Comment):
    class Meta:
        proxy = True


@receiver(pre_delete, sender=CommentReply)
def comment_reply_pre_delete_handler(sender, instance, **kwargs):
    """
    Deletes all generated reply comments.
    """
    instance.reply_comments.all().delete()


@receiver(m2m_changed, sender=CommentReply.replied_to_comments.through)
def comment_reply_post_create_handler(sender, instance, action, model, pk_set,
    using, **kwargs):
    if action == 'post_add':
        for replied_to_comment in instance.replied_to_comments.all():
            moderator_settings = getattr(settings, 'MODERATOR', None)
            offset_timedelta = timedelta(seconds=1)
            if moderator_settings:
                if 'REPLY_BEFORE_COMMENT' in moderator_settings:
                    if moderator_settings['REPLY_BEFORE_COMMENT']:
                        offset_timedelta = timedelta(seconds=-1)

            created = False
            # We use try except DoesNotExist instead of get or create to
            # allow us to add a is_reply_comment to a newly created comment
            # which facilitates realtime_comment_classifier below to distinguish
            # between normal comments and reply comments.
            try:
                comment_obj = Comment.objects.get(
                    content_type=replied_to_comment.content_type,
                    object_pk=replied_to_comment.object_pk,
                    site=replied_to_comment.site,
                    submit_date=replied_to_comment.submit_date + offset_timedelta,
                    user=instance.user,
                )
            except Comment.DoesNotExist:
                comment_obj = Comment(
                    content_type=replied_to_comment.content_type,
                    object_pk=replied_to_comment.object_pk,
                    site=replied_to_comment.site,
                    submit_date=replied_to_comment.submit_date + offset_timedelta,
                    user=instance.user,
                    comment=instance.comment_text,
                )
                comment_obj.is_reply_comment = True
                comment_obj.save()
                created = True

            if not created:
                comment_obj.comment = instance.comment_text
                comment_obj.save()

            if comment_obj not in instance.reply_comments.all():
                instance.reply_comments.add(comment_obj)


@receiver(post_save, sender=Comment)
def realtime_comment_classifier(sender, instance, created, **kwargs):
    """
    Classifies a comment after it has been created.

    This behaviour is configurable by the REALTIME_CLASSIFICATION MODERATOR,
    default behaviour is to classify(True).
    """
    # Only classify if newly created.
    if created:
        moderator_settings = getattr(settings, 'MODERATOR', None)
        if moderator_settings:
            if 'REALTIME_CLASSIFICATION' in moderator_settings:
                if not moderator_settings['REALTIME_CLASSIFICATION']:
                    return

        # Only classify if not a reply comment.
        if not getattr(instance, 'is_reply_comment', False):
            from moderator.utils import classify_comment
            classify_comment(instance)


@receiver(object_liked)
def flag_reported_comments(instance, request, **kwargs):
    if not getattr(instance, 'is_reply_comment', False):
        from moderator.tasks import flag_reported_comments_task
        flag_reported_comments_task.delay(instance)


# Enable voting on Comments (for negative votes/reporting abuse).
secretballot.enable_voting_on(Comment)
